# -*- coding: utf-8 -*-

# mapWOC
#
# Description: Weak abstractions of connections of a master to nodes.
#
# Authors:
# Felix Wolfsteller <felix.wolfsteller@intevation.de>
#
# Copyright:
# Copyright (C) 2011 Bundesamt für Sicherheit in der Informationstechnik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2,
# or, at your option, any later version as published by the Free
# Software Foundation
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

import ConfigParser
import json
import socket

from mapwoc.lib.mapwoc_json import mapwoc_from_json_hook
from mapwoc.lib.node import Node
from mapwoc.lib.trivialresponse import TrivialResponse

class NodeConnection(Node):
    """Data the Master need to know to query a Node."""

    def __init__ (self, filename) :
        """Read information about how to connecto to a node from a keyfile."""
        Node.__init__(self, filename) 

    def ask(self, question):
        """
        Connect to node, send a string and receive answer.
        @param question: Question to be asked.
        @return: Answer received.
        """

        try:
            # Create a socket (SOCK_STREAM means a TCP socket)
            sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

            # Connect to server and send data
            sock.connect((self.ip, int(self.port)))
            sock.send(question + "\r\n")

            # TODO handle all the odds of sockets, timeouts, exceptions, etc
            # TODO lessen assumption about amount of data transmitted (twisted?)

            # Receive data from the server and shut down
            received = sock.recv(16384)
            sock.close()
        except socket.error, msg:
            print "Exception: while talking to node %s:%s Error: %s" % \
                    (self.ip, self.port, str(msg))
            print "Question was: %s " % question
            received = "Connection Error: " + str(msg)

        return received

    def ask_decode(self, question):
        """
        Asks question to node, returns deserialized answer.
        """
        answer = self.ask(question)
        try:
            dec = json.loads(answer, object_hook=mapwoc_from_json_hook)
            # TODO make the hook also decode primitives.
            if dec is None:
                return json.loads(answer)
            else:
                return dec
        except Exception as e:
            print("panic: communication with node failed: (%s) %s "\
                    % (str(answer),str(e)))
            # TODO handle error
           # raise

    def ask_ok_tr(self, question):
        """
        Asks node for a trivial response and returns answer.
        @param question: Question to ask.
        @return: A pair, where the first element is True if no error is
                 indicated by the TrivialResponse, which is the second
                 element in the pair.
        """
        tr = self.ask_decode(question)
        ok = isinstance(tr, TrivialResponse) and not tr.error 
        return ok,tr
