# -*- coding: utf-8 -*-

# mapWOC
#
# Description: Process to poll a Redirector.
#
# Authors:
# Felix Wolfsteller <felix.wolfsteller@intevation.de>
#
# Copyright:
# Copyright (C) 2011 Bundesamt für Sicherheit in der Informationstechnik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2,
# or, at your option, any later version as published by the Free
# Software Foundation
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

import json
import socket
from time import sleep

from mapwoc.lib.mapwoc_json import mapwoc_from_json_hook as json_decode
from mapwoc.lib.tickprocess import TickProcess

# TODO use logging

# TODO Merge with other socket-talk implementations.
def ask_redirector(question, host, port):
    """
    Asks a redirector and returns the answer as a string.

    @param question: The Question to ask.
    @param host: The host where the redirector resides.
    @param port: The port of the redirector (ctrl).
    @return: Answer as string (empty in case of error).
    """

    try:
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        sock.connect((host,port))
        sock.send(question)
        received = sock.recv(32768)
        sock.close()
    except socket.error, msg:
        #logger.warning("Error when talking to redirector: %s" % msg)
        print "Redirector poll process got no answer. From %s:%d" % \
                (host,port)
        return ''
    return received


def ask_redirector_decode(question, host, port):
    """
    Asks a redirector, returns json-decoded answer.

    @param question: The Question to ask.
    @param host: The host where the redirector resides.
    @param port: The port of the redirector (ctrl).
    @return: Decoded object.
    """

    return json.loads(ask_redirector(question, host, port),
                      object_hook=json_decode)


class RedirectorPollProcess(TickProcess):
    """
    Ask a redirector every poll_interval seconds about its state.
    Its answer is put in the Queue.
    """

    def __init__(self, queue, poll_interval=60, host='localhost', port=8228):
        """
        Sets up (but doesnt start) the Process to periodically check for
        a redirectors state.

        @param queue: A Queue that will fill up with answers (strings) of the
                      redirector.
        @oaram poll_interval: The interval in seconds to wait between two
                              communication attempts to the redirector.
        @param host: The address at which the redirectorctlserver listens.
        @param port: The port at which the redirectorctlserver listens.
        """

        TickProcess.__init__(self, queue, poll_interval)
        self.host = host
        self.port = port

    def tick(self):
        """
        Ask redirector about its state, put answer string in queue, sleep
        given number of seconds.
        """

        # TODO try/catch here instead of in ask_redirector
        # TODO proper error handling
        response = ask_redirector_decode('state_ext 30 10', self.host, self.port)
        self.put(response)

